
% Simulate a two-state model of gene expression
import Gillespie.*
tic
%% Reaction network:


%% Rate constants
    p.alp0= 154.665;
    p.tm= 7.2*60;
    p.tB= 4.3*60;
    p.tE= 43*60;
    p.tc= 17*10;
    p.transB= 10/60;
    p.transE= 0.3/60;
    p.kb= 3.8/60;
    p.KDB2E= 0.3;
    p.KDB2E2=0.3;
    p.KD1= 10;
    p.KD2= 0.04;
    p.KD3= 30;
    p.kc= 2.0/60;

%% Initial state
N=20; %number of cells to run the simulation over
tspan = [0, 10000]; %seconds
x0    = [0, 200, 44, 0, 0, 4, 0, 0, 0];     %mRNA, protein
timer = tspan(1,1):0.5:tspan(1,2);

%% Specify reaction network
pfun = @propensities_2state;
stoich_matrix = [ 1  0  0  0  0  0  0  0  0    % m
                 -1  0  0  0  0  0  0  0  0     
                  0  1  0  0  0  0  0  0  0    %Bf
                  0 -1  0  0  0  0  0  0  0
                  0  0  2  0  0  0  0  0  0    %Ef
                  0  0  1  0  0  0  0  0  0
                  0  0 -1  0  0  0  0  0  0
                  0  0  0  1  0  0  0  0  0    %B2E
                  0  0  0 -1  0  0  0  0  0
                  0  0  0  0  1  0  0  0  0    %B2E2
                  0  0  0  0 -1  0  0  0  0
                  0 -1  0  0  0 -1  1  0  0    %O and Bf
                  0  1  0  0  0  1 -1  0  0
                  0  0  0 -1  0 -1  0  1  0    %O B2E
                  0  0  0  1  0  1  0 -1  0
                  0  0  0 -1  0  0  0 -1  1    %O.B2E and B2E
                  0  0  0  1  0  0  0  1 -1
                  0  0 -1  0  1  1  0 -1  0    %Ef and O.B2E
                  0  0  1  0 -1 -1  0  1  0
                  0  0 -1  0  1  0  0  1 -1    %Ef and (O.B2E)2
                  0  0  1  0 -1  0  0 -1  1
                  ];


%% Run simulation
X=zeros(N,size(stoich_matrix,2),size(timer,2));
for i=1:N
[t,x] = directMethod(stoich_matrix, pfun, tspan, x0, p);
%[t,x] = firstReactionMethod(stoich_matrix, pfun, tspan, x0, p);
xg = pchip(t,x',timer);
X(i,:,:)=xg;
end
%% Plot time course
X_mean = squeeze(mean(X));
figure();
stairs(timer,X_mean'); set(gca,'XLim',tspan);
xlabel('time (s)');
ylabel('molecules');
legend({'m','Bf','Ef','B2E','B2E2','O','O.bf','O.B2E','O.(B2E)2'});



%figure(2)
%hold on
 %for b=1:N
  % plot(timer,squeeze(X(b,:,:)))
 %end
%plot(timer, X_mean, 'k','LineWidth',1)
toc




function a = propensities_2state(x, p)
% Return reaction propensities given current state x
m      = x(1);
Bf     = x(2);
Ef     = x(3);
B2E    = x(4);
B2E2   = x(5);
O      = x(6);
OnBf   = x(7);
OnB2E  = x(8);
O2nB2E = x(9);

%r1= B2E*p.KDB2E*p.kb;     %decay of B2E
%r2= B2E2./p.tc;         %decay of B2E2 to Ef
%r3= B2E2.*p.KDB2E2*p.kb;   %decay of B2E2 to B2E  
%r4= B2E./p.tc;         %decay of B2E to Ef
%r5= Bf.*Ef.*p.kb;
a = [O.*p.alp0./4 ;                                                 %creation of m
     p.kc.*m.*Ef+m./p.tm;                                           %decay of m                     Something seems off here as well
     p.transB.*m+B2E*p.KDB2E*p.kb;                                  %creation of Bf
     Bf./p.tB+Bf.*Ef.*p.kb;                                         %decay of Bf                    Discuss missong terms with someone
     B2E2./p.tc;                                                    %double creation of Ef from E2B2
     p.transE.*m+B2E*p.KDB2E*p.kb+B2E2.*p.KDB2E2*p.kb+B2E./p.tc;    %single creation of Ef 
     Ef./p.tE+Bf.*Ef.*p.kb;                                         %decay of Ef
     Bf.*Ef.*p.kb+B2E2.*p.KDB2E2*p.kb;                              %creation of B2E
     B2E*p.KDB2E*p.kb+B2E./p.tc;                                    %deacy of B2E
     B2E.*Ef.*p.kb;                                                 %creation of B2E2
     B2E2./p.tc+B2E2.*p.KDB2E2*p.kb;                                %decay of B2E2
     p.kb.*Bf.*O;                                                   %binding with Bf            Operator from here down
     p.KD1.*p.kb.*OnBf;                                             %dissociation Bf
     p.kb.*O.*B2E;                                                  %binding with B2E
     p.KD3.*p.kb.*OnB2E;                                            %dissociation B2E
     p.kb.*OnB2E.*B2E;                                              %binding with 2nd B2E
     p.KD2.*p.kb.*O2nB2E.*B2E;                                      %dissociation 2nd B2E
     p.kb.*Ef.*OnB2E;                                               %stripping of B2E
     p.kb.*(p.KDB2E2./p.KD3).*O.*B2E2;                              %reverse ^
     p.kb.*Ef.*O2nB2E.*B2E;                                         %stripping of 2nd B2e
     p.kb.*(p.KDB2E2./p.KD2).*O.*B2E2];                             %reverse ^
end