
% Simulate a two-state model of gene expression
import Gillespie.*
tic
%% Reaction network:


%% Rate constants
    p.alp0= 154.665;
    p.tm= 7.2;
    p.tB= 4.3;
    p.tE= 43;
    p.tc= 17;
    p.transB= 10;
    p.transE= 0.3;
    p.kb= 3.8;
    p.KDB2E= 0.3;
    p.KDB2E2=0.3;
    p.KD1= 10;
    p.KD2= 0.04;
    p.KD3= 30;
    p.kc= 2.0;

%% Initial state
N=2; %number of cells to run the simulation over
tspan = [0, 400]; %minutes
x0    = [0, 20, 10, 30, 100, 4, 0, 0, 0];     %mRNA, protein
timer = tspan(1,1):0.5:tspan(1,2);

%% Specify reaction network
pfun = @propensities_2state;
stoich_matrix = [ 1  0  0  0  0  0  0  0  0    %1 m
                 -1  0  0  0  0  0  0  0  0     
                 -1  0  0  0  0  0  0  0  0
                  0  1  0  0  0  0  0  0  0    %4 Bf
                  0 -1  0  0  0  0  0  0  0    
                  0  0  1  0  0  0  0  0  0    %6 Ef
                  0  0 -1  0  0  0  0  0  0
                  0 -1 -1  1  0  0  0  0  0    %8 B2E
                  0  1  1 -1  0  0  0  0  0    % dissociation
                  0  0  1 -1  0  0  0  0  0    % decay of relb
                  0  0 -1 -1  1  0  0  0  0    %11 B2E2
                  0  0  1  1 -1  0  0  0  0    %
                  0  0  2  0 -1  0  0  0  0    %13 decay
                  0 -1  0  0  0 -1  1  0  0    %O and Bf
                  0  1  0  0  0  1 -1  0  0
                  0  0  0 -1  0 -1  0  1  0    %16 O B2E
                  0  0  0  1  0  1  0 -1  0
                  0  0  0 -1  0  0  0 -1  1    %18 O.B2E and B2E
                  0  0  0  1  0  0  0  1 -1
                  0  0 -1  0  1  1  0 -1  0    %20 Ef and O.B2E
                  0  0  1  0 -1 -1  0  1  0
                  0  0 -1  0  1  0  0  1 -1    %22 Ef and (O.B2E)2
                  0  0  1  0 -1  0  0 -1  1
                  ];


%% Run simulation
X=zeros(N,size(stoich_matrix,2),size(timer,2));
for i=1:N
[t,x] = directMethod(stoich_matrix, pfun, tspan, x0, p);
%[t,x] = firstReactionMethod(stoich_matrix, pfun, tspan, x0, p);
xg = pchip(t,x',timer);
X(i,:,:)=xg;
end
%% Plot time course
X_mean = squeeze(mean(X));
figure();
stairs(timer,X_mean'); set(gca,'XLim',tspan);
%stairs(t,x); set(gca,'XLim',tspan);
xlabel('time (s)');
ylabel('molecules');
legend({'m','Bf','Ef','B2E','B2E2','O','O.bf','O.B2E','O.(B2E)2'});



%figure(2)
%hold on
 %for b=1:N
  % plot(timer,squeeze(X(b,:,:)))
 %end
%plot(timer, X_mean, 'k','LineWidth',1)
toc




function a = propensities_2state(x, p)
% Return reaction propensities given current state x
m      = x(1);
Bf     = x(2);
Ef     = x(3);
B2E    = x(4);
B2E2   = x(5);
O      = x(6);
OnBf   = x(7);
OnB2E  = x(8);
O2nB2E = x(9);

%r1= B2E*p.KDB2E*p.kb;     %decay of B2E
%r2= B2E2./p.tc;         %decay of B2E2 to Ef
%r3= B2E2.*p.KDB2E2*p.kb;   %decay of B2E2 to B2E  
%r4= B2E./p.tc;         %decay of B2E to Ef
%r5= Bf.*Ef.*p.kb;
a = [O.*p.alp0./4 ;                                                 %1 translation of m
     m./p.tm;                                                       %2 decay of m                     Something seems off here as well
     p.kc.*m.*Ef;                                                   %3 degradation of m
     p.transB.*m;                                                   %4 translation Bf
     Bf./p.tB;                                                      %5 decay of Bf                    Discuss missong terms with someone
     p.transE.*m;                                                   %6 translation of Ef 
     Ef./p.tE;                                                      %7 decay of Ef
     Bf.*Ef.*p.kb;                                                  %8 combination of B2E
     B2E*p.KDB2E*p.kb;                                              %9 dissociation of B2E
     B2E./p.tc;                                                     %10 deacy of B2E
     B2E.*Ef.*p.kb;                                                 %11 combination of B2E2
     B2E2.*p.KDB2E2*p.kb;                                           %12 dissociation 
     B2E2./p.tc;                                                    %13 decay of B2E2
     p.kb.*Bf.*O;                                                   %14 binding with Bf            Operator from here down
     p.KD1.*p.kb.*OnBf;                                             %15 dissociation Bf
     p.kb.*O.*B2E;                                                  %16 binding with B2E
     p.KD3.*p.kb.*OnB2E;                                            %17 dissociation B2E
     p.kb.*OnB2E.*B2E;                                              %18 binding with 2nd B2E
     p.KD2.*p.kb.*O2nB2E.*B2E;                                      %19 dissociation 2nd B2E
     p.kb.*Ef.*OnB2E;                                               %20 stripping of B2E
     p.kb.*(p.KDB2E2./p.KD3).*O.*B2E2;                              %21 reverse ^
     p.kb.*Ef.*O2nB2E.*B2E;                                         %22 stripping of 2nd B2e
     p.kb.*(p.KDB2E2./p.KD2).*OnB2E*B2E2];                          %23 reverse ^
end