
% Simulate a two-state model of gene expression
import Gillespie.*
tic
%% Reaction network:


%% Rate constants
    p.alp0= 154.665;
    p.tm= 7.2;
    p.tB= 4.3;
    p.tE= 43;
    p.tc= 17;
    p.transB= 15;
    p.transE= 0.3;
    p.kb= 3.8;
    p.KDB2E= 0.3;
    p.KDB2E2=0.3;
    p.KD1= 10;
    p.KD2= 0.04;
    p.KD3= 30;
    p.kc= 2.0;

%% Additional parameters
p.PE1 = [0.01 0.03 0.1 0.3 1];          % promoter strength
p.LE = [1, 0.1 0.01 0.001];       % leaky factor
p.np = 350 ;       % plasmids in cell (ma be used as: instances of promoter)
p.p1 = [0,1];       % on off switch 
    
%% Initial state
N=1000; %number of cells to run the simulation over
tspan = [0, 120]; %minutes
x0    = [7, 410, 0, 65, 11, 0, 2, 0, 2];     %mRNA, protein
timer = tspan(1,1):0.5:tspan(1,2);


%% Specify reaction network
pfun = @propensities_2state;
stoich_matrix = [ 1  0  0  0  0  0  0  0  0    %1 m
                 -1  0  0  0  0  0  0  0  0     
                 -1  0  0  0  0  0  0  0  0
                  0  1  0  0  0  0  0  0  0    %4 Bf
                  0 -1  0  0  0  0  0  0  0    
                  0  0  1  0  0  0  0  0  0    %6 Ef
                  0  0 -1  0  0  0  0  0  0
                  0 -1 -1  1  0  0  0  0  0    %8 B2E
                  0  1  1 -1  0  0  0  0  0    % dissociation
                  0  0  1 -1  0  0  0  0  0    % decay of relb in B2E
                  0  0  0 -1  0  0  0  0  0
                  0  0 -1 -1  1  0  0  0  0    %12 B2E2
                  0  0  1  1 -1  0  0  0  0    %
                  0  0  2  0 -1  0  0  0  0    %14 decay of relB
                  0  0  0  0 -1  0  0  0  0  
                  0 -1  0  0  0 -1  1  0  0    %16 O and Bf
                  0  1  0  0  0  1 -1  0  0
                  0  0  0 -1  0 -1  0  1  0    %18 O B2E
                  0  0  0  1  0  1  0 -1  0
                  0  0  0 -1  0  0  0 -1  1    %20 O.B2E and B2E
                  0  0  0  1  0  0  0  1 -1
                  0  0 -1  0  1  1  0 -1  0    %22 Ef and O.B2E
                  0  0  1  0 -1 -1  0  1  0
                  0  0 -1  0  1  0  0  1 -1    %24 Ef and (O.B2E)2
                  0  0  1  0 -1  0  0 -1  1
                  0  0  1  0  0  0  0  0  0    % OUR relE promoter
                  ];



 %% Run simulation
p.PE=p.PE1(1);
 p.p=p.p1(1);
 for u=1:N
     disp(sprintf('Preliminary %g/%g',u,N));
[t1,x] = directMethod(stoich_matrix, pfun, [0 30], x0, p);
%[t,x] = firstReactionMethod(stoich_matrix, pfun, tspan, x0, p);
X0(u,:)= x(end,:);
 end
  
  
X=zeros(N,size(stoich_matrix,2),size(timer,2));
 p.p=p.p1(2);
 n=0;
for j=1:size(p.PE1,2)
 p.PE=p.PE1(j);
for i=1:N
 disp(sprintf('Calculating species for cell %g/%g Part %g/%g',i,N,j,size(p.PE1,2)));
[t,x] = directMethod(stoich_matrix, pfun, tspan, X0(i,:), p);
%[t,x] = firstReactionMethod(stoich_matrix, pfun, tspan, x0, p);
xg = pchip(t,x',timer);  
X(i,:,:)=xg;
end
n=n+1;
xlswrite('Xdata.xlsx',squeeze(X(:,3,:)),n);

end
%% Plot time course
X_mean = squeeze(mean(X));
figure();
stairs(timer,X_mean'); set(gca,'XLim',tspan);
%stairs(t,x); set(gca,'XLim',tspan);
%semilogy(timer,X_mean)
xlabel('time (s)');
ylabel('molecules');
legend({'m','B2','Ef','B2E','B2E2','O','O.bf','O.B2E','O.(B2E)2'});



%figure(2)
%hold on
 %for b=1:N
  % plot(timer,squeeze(X(b,:,:)))
 %end
%plot(timer, X_mean, 'k','LineWidth',1)
toc


%% Save to Exel


%%
function a = propensities_2state(x, p)
% Return reaction propensities given current state x
m      = x(1);
Bf     = x(2);
Ef     = x(3);
B2E    = x(4);
B2E2   = x(5);
O      = x(6);
OnBf   = x(7);
OnB2E  = x(8);
O2nB2E = x(9);

%r1= B2E*p.KDB2E*p.kb;     %decay of B2E
%r2= B2E2./p.tc;         %decay of B2E2 to Ef
%r3= B2E2.*p.KDB2E2*p.kb;   %decay of B2E2 to B2E  
%r4= B2E./p.tc;         %decay of B2E to Ef
%r5= Bf.*Ef.*p.kb;
a = [O.*p.alp0./4 ;           %1 translation of m
     m./p.tm;                              %2 decay of m                     Something seems off here as well
     p.kc.*m.*Ef;                        %3 degradation of m
     p.transB.*m;                         %4 translation Bf
     Bf./p.tB;                              %5 decay of Bf                    Discuss missong terms with someone
     p.transE.*m;                         %6 translation of Ef 
     Ef./p.tE;                          %7 decay of Ef
     Bf.*Ef.*p.kb;                       %8 combination of B2E
     B2E*p.KDB2E*p.kb;                    %9 dissociation of B2E
     B2E./p.tc;                         %10 deacy of ewlb in b2e
     B2E/p.tE;                          %11 decay
     B2E.*Ef.*p.kb;                       %12 combination of B2E2
     B2E2.*p.KDB2E2*p.kb;                 %13 dissociation 
     B2E2./p.tc;                        %14 decay of relB in b2e2
     B2E2./p.tE;                        %15 decay
     p.kb.*Bf.*O;                       %16 binding with Bf            Operator from here down
     p.KD1.*p.kb.*OnBf;                     %17 dissociation Bf
     p.kb.*O.*B2E;                        %18 binding with B2E
     p.KD3.*p.kb.*OnB2E;                   %19 dissociation B2E
     p.kb.*OnB2E.*B2E;                       %20 binding with 2nd B2E
     p.KD2.*p.kb.*O2nB2E.*B2E;             %21 dissociation 2nd B2E
     p.kb.*Ef.*OnB2E;                       %22 stripping of B2E
     p.kb.*(p.KDB2E2./p.KD3).*O.*B2E2;      %23 reverse ^
     p.kb.*Ef.*O2nB2E.*B2E;                 %24 stripping of 2nd B2e
     p.kb.*(p.KDB2E2./p.KD2).*OnB2E*B2E2;  %25 reverse ^
     p.PE.*p.np*p.p;                          % inserted promoter
     ];                               
end